@@ -18,6 +18,7 @@
 #include "public/adios.h"
 #include "public/adios_error.h"
 #include "core/globals.h"
+#include "core/buffer.h"
 #include "core/common_adios.h"
 #include "core/adios_bp_v1.h"
 #include "core/adios_internals.h"
@@ -45,6 +46,10 @@ int adios_init (const char * config, MPI_Comm comm)
     return common_adios_init (config, comm);
 }
 
+int adios_is_initialized(void)
+{
+    return (adios_transports != NULL);
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 // all XML file pieces will be provided by another series of calls
@@ -66,6 +71,13 @@ int adios_allocate_buffer (enum ADIOS_BUFFER_ALLOC_WHEN adios_buffer_alloc_when
     return common_adios_allocate_buffer (adios_buffer_alloc_when, buffer_size);
 }
 
+///////////////////////////////////////////////////////////////////////////////
+void adios_set_max_buffer_size (uint64_t max_buffer_size_MB)
+{
+    if (max_buffer_size_MB > 0)
+        adios_databuffer_set_max_size (max_buffer_size_MB * 1024L * 1024L);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 int adios_open (int64_t * fd, const char * group_name, const char * name
                ,const char * mode, MPI_Comm comm
@@ -125,95 +137,6 @@ int adios_write (int64_t fd_p, const char * name, const void * var)
     }
 
     retval = common_adios_write_byid (fd, v, var);
-#if 0
-    if (fd->mode == adios_mode_read)
-    {
-        if (   strcasecmp (name, fd->group->group_comm)
-            && v->is_dim != adios_flag_yes
-           )
-        {
-            adios_error (err_invalid_file_mode, "write attempted on %s in %s.  This was opened for read\n" ,name , fd->name);
-            return 1;
-        }
-    }
-
-    if (!var)
-    {
-        adios_error (err_invalid_data, "Invalid data (NULL pointer) passed to write for variable %s\n", name);
-
-        return 1;
-    }
-
-    if (v->data)
-    {
-        free (v->data);
-        v->data = 0;
-    }
-
-    if (v->dimensions)
-    {
-        v->data = var;
-    }
-    else
-    {
-        uint64_t element_size = adios_get_type_size (v->type, var);
-
-        switch (v->type)
-        {
-            case adios_byte:
-            case adios_short:
-            case adios_integer:
-            case adios_long:
-            case adios_unsigned_byte:
-            case adios_unsigned_short:
-            case adios_unsigned_integer:
-            case adios_unsigned_long:
-            case adios_real:
-            case adios_double:
-            case adios_long_double:
-            case adios_complex:
-            case adios_double_complex:
-                v->data = malloc (element_size);
-                if (!v->data)
-                {
-                    adios_error (err_no_memory, 
-                                 "In adios_write, cannot allocate %lld bytes to copy scalar %s\n",
-                                 element_size, v->name);
-
-                    return 0;
-                }
-
-                memcpy ((char *) v->data, var, element_size);
-                break;
-
-            case adios_string:
-                v->data = malloc (element_size + 1);
-                if (!v->data)
-                {
-                    adios_error (err_no_memory, 
-                                 "In adios_write, cannot allocate %lld bytes to copy string %s\n",
-                                 element_size, v->name);
-
-                    return 0;
-                }
-                ((char *) v->data) [element_size] = 0;
-                memcpy ((char *) v->data, var, element_size);
-                break;
-
-            default:
-                v->data = 0;
-                break;
-        }
-    }
-
-    common_adios_write (fd, v, var);
-    // v->data is set to NULL in the above call
-
-    if (fd->mode == adios_mode_write || fd->mode == adios_mode_append) 
-    {
-        adios_copy_var_written (fd->group, v);
-    }
-#endif
     return retval;
 }
 
@@ -280,7 +203,7 @@ int adios_close (int64_t fd_p)
     }
     struct adios_var_struct * v = fd->group->vars;
 
-    retval = common_adios_close (fd_p);
+    retval = common_adios_close (fd);
 
     // Q.L. 10-2010. To fix a memory leak problem.
     while (v) {
@@ -340,7 +263,7 @@ int adios_close (int64_t fd_p)
 // group a list of vars into a composite group
 int adios_declare_group (int64_t * id, const char * name
                         ,const char * time_index
-                        ,enum ADIOS_FLAG stats
+                        ,enum ADIOS_STATISTICS_FLAG stats
                         )
 {
     int ret;
@@ -349,7 +272,7 @@ int adios_declare_group (int64_t * id, const char * name
                                       ,""
                                       ,""
                                       ,time_index
-                                      ,stats
+                                      ,(stats != adios_stat_no_do_not_use_this ? stats : adios_stat_no)
                                       );
     if (ret == 1) {
         struct adios_group_struct * g = (struct adios_group_struct *) *id;
@@ -365,6 +288,27 @@ int adios_free_group (int64_t id)
     return adios_errno;
 }
 
+
+
+int adios_set_time_aggregation(int64_t groupid,
+                               uint64_t buffersize,
+                               int64_t syncgroupid
+                              )
+{
+    adios_errno = err_no_error;
+    if (groupid == 0) {
+        adios_error (err_invalid_group, "adios_set_time_aggregation() called with 0 argument\n");
+    }
+    struct adios_group_struct * g = (struct adios_group_struct *) groupid;
+    struct adios_group_struct * sg = NULL;
+    if (syncgroupid != 0)
+        sg = (struct adios_group_struct *) syncgroupid;
+
+    adios_common_set_time_aggregation(g, buffersize, sg);
+    return adios_errno;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 // adios_common_define_var is in adios_internals.c
@@ -400,6 +344,51 @@ int adios_delete_vardefs (int64_t id)
     return adios_errno;
 }
 
+// Return the expected size (in bytes) of a defined variable
+// It is simply the product of local dimensions and byte-size of type
+uint64_t adios_expected_var_size (int64_t var_id)
+{
+    adios_errno = err_no_error;
+    uint64_t size = 0;
+    if (var_id != 0) {
+        struct adios_var_struct * var = (struct adios_var_struct *) var_id;
+        if (var->transform_type != adios_transform_none) {
+            size = adios_transform_get_pre_transform_var_size (var);
+        } else {
+            size = adios_get_var_size (var, var->data);
+        }
+        if (size == 0 || adios_errno != err_no_error) {
+            if (adios_errno == err_invalid_var_as_dimension)
+            {
+                log_error ("%s: An array size depends on the actual value of the dimension variable. "
+                         "This will be known after adios_write() of that dimension variable.\n",
+                         __func__);
+            }
+        }
+        /*
+            enum ADIOS_DATATYPES original_var_type = adios_transform_get_var_original_type_var (var);
+            size = adios_get_type_size(original_var_type, NULL);
+            struct adios_dimension_struct * d = var->dimensions;
+            int i=1;
+            while (d)
+            {
+                size *= adios_get_dim_value (&d->dimension);
+                if (size == 0 || adios_errno != err_no_error) {
+                    adios_error (err_dimension_required, "%s: the %d. dimension of variable %s is not yet known. "
+                                 "The dimension will be known after adios_write() of that dimension variable\n",
+                                 __func__, i, var->name);
+                }
+                d = d->next;
+                i++;
+            }
+         */
+    } else {
+        adios_error (err_invalid_varid, "%s called with invalid variable ID\n", __func__);
+    }
+
+    return size;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // adios_common_set_transform is in adios_internals.c
 // set the transform method for the selected variable (default is "none")
@@ -440,7 +429,7 @@ int adios_delete_attrdefs (int64_t id)
 
 int adios_define_attribute_byvalue (int64_t group, 
                             const char * name, const char * path, 
-                            enum ADIOS_DATATYPES type, int  nelems, void * values
+                            enum ADIOS_DATATYPES type, int  nelems, const void * values
                            )
 {
     adios_errno = err_no_error;
@@ -609,3 +598,53 @@ int adios_define_mesh_unstructured (char * points,
     return adios_common_define_mesh_unstructured (points, data, count, cell_type, nspace, npoints, name, group_id);
 }
 
+ADIOS_AVAILABLE_WRITE_METHODS * adios_available_write_methods()
+{
+    int i, n;
+    n = 0;
+    for (i = 0; i < ADIOS_METHOD_COUNT; i++) {
+        if (adios_transports[i].method_name) {
+            n++;
+        }
+    }
+
+    if (n == 0)
+        return NULL;
+
+    ADIOS_AVAILABLE_WRITE_METHODS * m = (ADIOS_AVAILABLE_WRITE_METHODS *) malloc (sizeof(ADIOS_AVAILABLE_WRITE_METHODS));
+    if (!m)
+        return NULL;
+
+    m->name     = (char**) malloc (n*sizeof(char*));
+    m->nmethods = n;
+
+    n = 0;
+    for (i = 0; i < ADIOS_METHOD_COUNT; i++) {
+        if (adios_transports[i].method_name) {
+            m->name[n] = strdup (adios_transports[i].method_name);
+            n++;
+        }
+    }
+    return m;
+}
+
+void adios_available_write_methods_free (ADIOS_AVAILABLE_WRITE_METHODS * m)
+{
+	int i;
+	if (m)
+	{
+	    if (m->name)
+	    {
+	        for (i=0; i < m->nmethods; i++)
+	        {
+	            if (m->name[i]) {
+	                free (m->name[i]);
+	                m->name[i] = NULL;
+	            }
+	        }
+	        free (m->name);
+	        m->name = NULL;
+	    }
+	    free (m);
+	}
+}
