@@ -9,6 +9,7 @@
 #define ADIOS_INTERNALS_H
 
 #include <stdint.h>
+#include <inttypes.h>
 #include <stdlib.h>
 
 // need the enum for the transports
@@ -16,6 +17,8 @@
 #include "core/adios_transport_hooks.h"
 #include "core/adios_bp_v1.h"
 #include "core/qhashtbl.h"
+#include "core/types.h"
+#include "core/strutil.h" /* PairStruct* */
 #include "public/adios_schema.h"
 
 // NCSU ALACRITY-ADIOS: Include needed for the transform spec struct
@@ -144,14 +147,12 @@ struct adios_group_struct
     struct adios_var_struct * vars;
     struct adios_var_struct * vars_tail;  // last variable in the list 'vars'
     qhashtbl_t *hashtbl_vars;
-    struct adios_var_struct * vars_written;
-    struct adios_var_struct * vars_written_tail; // last variable in 'vars_written'
     struct adios_attribute_struct * attributes;
     char * group_comm;
     char * group_by;
     char * time_index_name;
     uint32_t time_index;
-    enum ADIOS_FLAG stats_on;
+    enum ADIOS_STATISTICS_FLAG stats_flag;
     uint32_t process_id;
 
     struct adios_method_list_struct * methods;
@@ -161,15 +162,118 @@ struct adios_group_struct
     enum ADIOS_FLAG all_unique_mesh_names;
 
     int attrid_update_epoch; // ID of special attribute "/__adios__/update_time_epoch" to find it fast
+    uint64_t last_buffer_size; // remember how much buffer we used in previous output steps
 
 #if defined ADIOS_TIMERS || defined ADIOS_TIMER_EVENTS
     // Using a "double buffering" approach. Current write cycle stored in timing_obj, while timing info from
     // previous cycle is kept in prev_timing_obj, and is written before close
     struct adios_timing_struct * timing_obj;
     struct adios_timing_struct * prev_timing_obj;
+    uint64_t tv_size; // the additional data size used by timing variables
 #endif
+    int do_ts_aggr; //Yuan: introduced for time steps buffering
+    struct adios_file_struct *ts_fd; // save and keep open the file struct during time aggr.
+    uint64_t ts_buffsize; //Yuan: introduced for time steps buffering
+    int ts_to_buffer; //current time steps
+    int max_ts; //maximum time steps to buffer 
+    struct adios_index_struct_v1 * index; //the indexes for current written PGs 
+    int built_index; // FIXME: 0 or 1, if index has been built, do not build it during close()
+    int do_ts_flush; // 1 if we need close during finalize/group-sync, otherwise always 0
+    struct adios_group_struct **synced_groups;
+    int synced_groups_size; // > 0 if this group forces time-aggregated groups to be flushed
+    int synced_groups_capacity; // synced_groups is a Vector
 };
 
+static inline void SetTimeAggregation (struct adios_group_struct * g, int flag)
+{
+    g->do_ts_aggr = (flag != 0);
+}
+
+
+static inline int TimeAggregated (struct adios_group_struct * g)
+{
+    return (g->do_ts_aggr != 0);
+}
+
+static inline int NotTimeAggregated (struct adios_group_struct * g)
+{
+    return (g->do_ts_aggr == 0);
+}
+
+static inline int TimeAggregationJustBegan (struct adios_group_struct * g)
+{
+    return (g->do_ts_aggr
+            && g->ts_fd == NULL
+            //&& g->ts_to_buffer == g->max_ts
+            );
+}
+
+static inline int TimeAggregationInProgress (struct adios_group_struct * g)
+{
+    return (g->do_ts_aggr
+            && g->ts_fd != NULL
+            //&& g->ts_to_buffer < g->max_ts
+            );
+}
+// TimeAggregationAggregationInProgress => TimeAggregated AND NOT TimeAggreationJustBegan
+// TimeAggregationAggregationJustBegan  => TimeAggregated AND NOT TimeAggreationInProgress
+
+static inline int TimeAggregationLastStep (struct adios_group_struct * g)
+{
+    return (g->do_ts_aggr  && g->ts_to_buffer == 0);
+}
+// TimeAggregationAggregationLastStep => TimeAggregated AND TimeAggreationInProgress
+
+static inline void SetTimeAggregationFlush (struct adios_group_struct * g, int do_flush)
+{
+    g->do_ts_flush = (do_flush != 0);  // => TimeAggregationIsFlushing
+    if (g->do_ts_flush)
+        g->ts_to_buffer = 0;               // => TimeAggregationLastStep
+}
+
+// TimeAggregationIsFlushing => TimeAggregated AND TimeAggreationInProgress AND TimeAggregationLastStep
+static inline int TimeAggregationIsFlushing (struct adios_group_struct * g)
+{
+    return (g->do_ts_flush != 0);
+}
+
+static inline void SetTimeAggregationSyncGroup (struct adios_group_struct * g,
+                                                struct adios_group_struct * synced_group)
+{
+    if (g->synced_groups_capacity <= g->synced_groups_size) // first synced group
+    {
+        struct adios_group_struct ** newg = (struct adios_group_struct **)
+                realloc (g->synced_groups, g->synced_groups_size+5);
+        if (newg != NULL)
+        {
+            g->synced_groups_capacity = g->synced_groups_size + 5;
+            g->synced_groups = newg;
+        }
+        /*else
+        {
+            adios_error(err_no_memory, "Could not (re)allocate small memory for synchronized groups. "
+                    "Keep the current list of %d groups synchronized with group %s\n",
+                    g->synced_groups_capacity, g->name);
+        }*/
+    }
+
+    g->synced_groups[g->synced_groups_size] = synced_group;
+    g->synced_groups_size++;  // => TimeAggregationIsaSyncGroup
+}
+
+static inline int TimeAggregationIsaSyncGroup (struct adios_group_struct * g)
+{
+    return (g->synced_groups_size != 0);
+}
+
+static inline void TimeAggregationGetSyncedGroups (struct adios_group_struct * g,
+                                                   struct adios_group_struct *** synced_groups,
+                                                   int *ngroups)
+{
+    *synced_groups = g->synced_groups;
+    *ngroups = g->synced_groups_size;
+}
+
 struct adios_group_list_struct
 {
     struct adios_group_struct * group;
@@ -177,31 +281,49 @@ struct adios_group_list_struct
     struct adios_group_list_struct * next;
 };
 
+/* List of PGs in a file struct to store variables related to each PG written
+   in one open()...close() cycle. If there is no buffer overflow, there will
+   be a single PG written 
+*/
+struct adios_pg_struct
+{
+    uint64_t pg_start_in_file; //  where this pg start in file, handled by methods!
+    uint64_t has_index; // Yuan: 0 or 1, if the index has been built for this PG 
+    struct adios_var_struct * vars_written;
+    struct adios_var_struct * vars_written_tail; // last variable in 'vars_written'
+    struct adios_pg_struct  * next;
+};
+
 struct adios_file_struct
 {
     char * name;
     int32_t subfile_index; // needs to be set in ADIOS method if generates subfiles
     struct adios_group_struct * group;
     enum ADIOS_METHOD_MODE mode;
-    uint64_t data_size;
-    uint64_t write_size_bytes;
 
-    enum ADIOS_FLAG shared_buffer;
+    enum ADIOS_FLAG shared_buffer; // yes: common layer does the buffering (in BP format)
+    enum BUFFERING_STRATEGY bufstrat; // how to do buffering (no_buffering <-> shared_buffer==adios_flag_no)
+    enum BUFFERING_STATE bufstate; // is buffering still going on (depends on overflow strategy)
 
-    uint64_t pg_start_in_file; //  where this pg started in the file
+    struct adios_pg_struct * pgs_written;
+    struct adios_pg_struct * current_pg; // points to last PG in the list, which is being created in buffer
+    struct adios_pg_struct * first_pg_written; // Yuan: points to first PG in the list
 
-    uint64_t base_offset;   // where writing last ocurred
-
-    char * buffer;          // buffer we use for building the output
+    char * allocated_bufptr;  // actual allocated buffer before alignment
+    char * buffer;          // buffer we use for building the output (aligned, made from allocated_bufptr)
     uint64_t offset;        // current offset to write at
-    uint64_t bytes_written; // largest offset into buffer written to
+    uint64_t bytes_written; // largest offset into buffer written to, = offset after calling _v1() functions
     uint64_t buffer_size;   // how big the buffer is currently
 
-    uint64_t vars_start;    // offset for where to put the var/attr count
-    uint32_t vars_written;  // count of vars/attrs to write
+    uint64_t pg_start;      // offset in buffer where to put PG size (it should be 0 to point to buffer[0])
+    uint64_t vars_start;    // offset for where to put the vars count
+    uint32_t nvars_written;  // count of vars to write
+    uint64_t attrs_start;    // offset for where to put the attr count
+    uint32_t nattrs_written;  // count of attrs to write
 
     MPI_Comm comm;          // duplicate of comm received in adios_open()
 };
+void adios_file_struct_init (struct adios_file_struct * fd);
 
 struct adios_dimension_item_struct
 {
@@ -321,7 +443,7 @@ typedef void (* ADIOS_INIT_FN) (const PairStruct * parameters
 typedef int (* ADIOS_OPEN_FN) (struct adios_file_struct * fd
                               ,struct adios_method_struct * method, MPI_Comm comm
                               );
-typedef enum ADIOS_FLAG (* ADIOS_SHOULD_BUFFER_FN)
+typedef enum BUFFERING_STRATEGY (* ADIOS_SHOULD_BUFFER_FN)
                                        (struct adios_file_struct * fd
                                        ,struct adios_method_struct * method
                                        );
@@ -342,6 +464,9 @@ typedef void (* ADIOS_READ_FN) (struct adios_file_struct * fd
                                ,uint64_t buffer_size
                                ,struct adios_method_struct * method
                                );
+typedef void (* ADIOS_BUFFER_OVERFLOW_FN) (struct adios_file_struct * fd 
+                                          ,struct adios_method_struct * method 
+                                          ); 
 typedef void (* ADIOS_CLOSE_FN) (struct adios_file_struct * fd
                                 ,struct adios_method_struct * method
                                 );
@@ -363,6 +488,7 @@ struct adios_transport_struct
     ADIOS_WRITE_FN adios_write_fn;
     ADIOS_GET_WRITE_BUFFER_FN adios_get_write_buffer_fn;
     ADIOS_READ_FN adios_read_fn;
+    ADIOS_BUFFER_OVERFLOW_FN adios_buffer_overflow_fn;
     ADIOS_CLOSE_FN adios_close_fn;
     ADIOS_FINALIZE_FN adios_finalize_fn;
     ADIOS_END_ITERATION_FN adios_end_iteration_fn;
@@ -440,7 +566,7 @@ int adios_common_define_attribute_byvalue (int64_t group, const char * name
                                   ,const char * path
                                   ,enum ADIOS_DATATYPES type
                                   ,int nelems
-                                  ,void * values
+                                  ,const void * values
                                   );
 
 void adios_append_method (struct adios_method_struct * method);
@@ -451,6 +577,9 @@ void adios_add_method_to_group (struct adios_method_list_struct ** root
 
 void adios_append_group (struct adios_group_struct * group);
 
+struct adios_pg_struct * add_new_pg_written (struct adios_file_struct * fd);
+void adios_free_pglist (struct adios_file_struct * fd);
+
 //void adios_append_var (struct adios_group_struct * g, struct adios_var_struct * var);
 
 void adios_append_dimension (struct adios_dimension_struct ** root
@@ -467,8 +596,13 @@ int adios_common_declare_group (int64_t * id, const char * name
                                ,const char * coordination_comm
                                ,const char * coordination_var
                                ,const char * time_index_name
-                               ,enum ADIOS_FLAG stats
-                               );
+                               ,enum ADIOS_STATISTICS_FLAG stats
+);
+
+int adios_common_set_time_aggregation(struct adios_group_struct * group,
+                                      uint64_t buffersize,
+                                      struct adios_group_struct * syncgroup
+);
 
 int64_t adios_common_define_var (int64_t group_id, const char * name
                                 ,const char * path, enum ADIOS_DATATYPES type
@@ -488,8 +622,10 @@ int adios_common_define_var_characteristics  (struct adios_group_struct * g
                                               ,const char * bin_count
                                              );
 
-void adios_common_get_group (int64_t * group_id, const char * name);
+struct adios_group_struct * adios_common_get_group (const char * name);
+int adios_common_delete_attrdefs (struct adios_group_struct * g);
 int adios_common_delete_vardefs (struct adios_group_struct * g);
+void adios_common_free_groupstruct (struct adios_group_struct * g);
 int adios_common_free_group (int64_t id);
 
 // ADIOS file format functions
@@ -497,6 +633,7 @@ int adios_common_free_group (int64_t id);
 uint16_t adios_calc_var_characteristics_dims_overhead(struct adios_dimension_struct * d);
 uint16_t adios_calc_var_overhead_v1 (struct adios_var_struct * v);
 uint32_t adios_calc_attribute_overhead_v1 (struct adios_attribute_struct * a);
+uint32_t adios_calc_attrs_overhead_v1 (struct adios_file_struct * fd);
 uint64_t adios_calc_overhead_v1 (struct adios_file_struct * fd);
 
 int adios_write_version_v1 (char ** buffer
@@ -508,11 +645,10 @@ int adios_write_version_flag_v1 (char ** buffer
                            ,uint64_t * buffer_offset
                            ,uint32_t flag
                            );
-int adios_write_process_group_header_v1 (struct adios_file_struct * fd
-                                        ,uint64_t total_size
-                                        );
+int adios_write_open_process_group_header_v1 (struct adios_file_struct * fd);
+int adios_write_close_process_group_header_v1 (struct adios_file_struct * fd);
 
-void adios_copy_var_written (struct adios_group_struct * g,
+void adios_copy_var_written (struct adios_file_struct * fd,
                              struct adios_var_struct * var);
 
 // data is only there for sizing
@@ -588,7 +724,10 @@ uint64_t adios_get_type_size (enum ADIOS_DATATYPES type, const void * var);
 // NCSU ALACRITY-ADIOS - added this for use in the transform layer
 uint64_t adios_get_dimension_space_size (struct adios_var_struct * var
                                         ,struct adios_dimension_struct * d);
+// get the size of variable data for output (i.e. possibly after transformation)
 uint64_t adios_get_var_size (struct adios_var_struct * var, const void * data);
+// get the size of original variable data if variable is transformed (call only if it's transformed)
+uint64_t adios_transform_get_pre_transform_var_size(struct adios_var_struct *var);
 uint64_t adios_get_dim_value (struct adios_dimension_item_struct * dimension);
 uint64_t adios_get_stat_size (void * data, enum ADIOS_DATATYPES type, enum ADIOS_STAT stat_id);
 uint8_t adios_get_stat_set_count (enum ADIOS_DATATYPES type);
@@ -608,6 +747,7 @@ int adios_int_is_var (const char * temp); // 1 == yes, 0 == no
 int adios_int_is_num (char * temp); // 1 == yes, 0 == no
 void adios_conca_mesh_numb_att_nam (char ** returnstr, const char * meshname, char * att_nam, char counterstr[5]);
 void adios_conca_mesh_att_nam (char ** returnstr, const char * meshname, char * att_nam);
+void adios_conca_link_att_nam(char ** returnstr, const char * name, char * att_nam, char counterstr[5]);
 
 // No-XML API
 int adios_common_define_schema_version (struct adios_group_struct * new_group, char * schema_version);
