@@ -461,7 +461,7 @@ ADIOS_SELECTION * new_derelativized_selection(const ADIOS_SELECTION *sel, const
         // Add the global offset to the bounding box start
         vector_add(ndim, new_start, sel->u.bb.start, sel_global_offset);
 
-        new_sel = common_read_selection_boundingbox(ndim, new_start, new_count);
+        new_sel = a2sel_boundingbox(ndim, new_start, new_count);
         break;
     }
     case ADIOS_SELECTION_POINTS:
@@ -481,7 +481,7 @@ ADIOS_SELECTION * new_derelativized_selection(const ADIOS_SELECTION *sel, const
             cur_dst_point += ndim;
         }
 
-        new_sel = common_read_selection_points(ndim, npoints, new_points);
+        new_sel = a2sel_points(ndim, npoints, new_points, NULL, 0);
         break;
     }
     case ADIOS_SELECTION_WRITEBLOCK:
@@ -497,7 +497,7 @@ ADIOS_SELECTION * new_derelativized_selection(const ADIOS_SELECTION *sel, const
 }
 
 ADIOS_SELECTION * varblock_to_bb(int ndim, const ADIOS_VARBLOCK *vb) {
-    return common_read_selection_boundingbox(ndim,
+    return a2sel_boundingbox(ndim,
                                              bufdup(vb->start, sizeof(uint64_t), ndim),
                                              bufdup(vb->count, sizeof(uint64_t), ndim));
 }
