@@ -8,6 +8,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdlib.h>
+#include <inttypes.h>
 #include <math.h>
 #include <string.h>
 #include <errno.h>
@@ -41,12 +42,13 @@
 // // evpath libraries
 #include <evpath.h>
 #include <cod.h>
+#define FLEXPATH_SIDE "WRITER"
 #include "core/flexpath.h"
 #include <sys/queue.h>
 
 /************************* Structure and Type Definitions ***********************/
 // used for messages in the control queue
-typedef enum {VAR=0, DATA_FLUSH, OPEN, CLOSE, INIT, EVGROUP_FLUSH, DATA_BUFFER} FlexpathMessageType;
+typedef enum {VAR=0, DATA_FLUSH, OPEN, CLOSE, INIT, EVGROUP_FLUSH, DATA_BUFFER, FINALIZE} FlexpathMessageType;
 
 // maintains connection information
 typedef struct _flexpath_stone {
@@ -156,6 +158,9 @@ typedef struct _flexpath_write_file_data {
     // for maintaining open file list
     struct _flexpath_write_file_data* next;
     char* name;
+
+    // general
+    int verbose;
 } FlexpathWriteFileData;
 
 typedef struct _flexpath_write_data {
@@ -325,7 +330,7 @@ data_free(void* eventData, void* clientData)
 void 
 op_free(void* eventData, void* clientData) 
 {
-    fp_write_log("OP", "freeing an op message\n");
+//    fp_write_log("OP", "freeing an op message\n");
     op_msg* op = (op_msg*) eventData;
     if (op->file_name) {
         free(op->file_name);
@@ -857,7 +862,7 @@ set_format(struct adios_group_struct *t,
 		    snprintf(el, ELSIZE, "[%s]", a->name);
 		    v_offset = 0;
 		} else {
-		    snprintf(el, ELSIZE, "[%llu]", d->dimension.rank);
+		    snprintf(el, ELSIZE, "[%" PRIu64 "]", d->dimension.rank);
 		    v_offset *= d->dimension.rank;
 		}
 		strncat(dims, el, DIMSIZE);
@@ -1027,7 +1032,7 @@ set_format(struct adios_group_struct *t,
 	    }
 	}
 
-	fp_write_log("FORMAT","field: %s, %s, %d, %d\n", 
+	fp_verbose(fileData, "field: %s, %s, %d, %d\n", 
 		     field_list[fieldNo].field_name, 
 		     field_list[fieldNo].field_type,
 		     field_list[fieldNo].field_size,
@@ -1108,6 +1113,7 @@ process_data_flush(FlexpathWriteFileData *fileData,
 		   Flush_msg *flushMsg, 
 		   FlexpathQueueNode *dataNode)
 {
+    //fprintf(stderr, "writer:%d:processing flush for reader:%d:reader_step:%d:writer_step:%d\n", fileData->rank, flushMsg->process_id, fileData->readerStep, fileData->writerStep);
     void* temp = copy_buffer(dataNode->data, flushMsg->process_id, fileData);
    
     fileData->attrs = set_dst_rank_atom(fileData->attrs, flushMsg->process_id);
@@ -1119,20 +1125,44 @@ process_data_flush(FlexpathWriteFileData *fileData,
     }
     //EVsubmit_general(fileData->dataSource, temp, data_free, fileData->attrs);
     EVsubmit_general(fileData->dataSource, temp, NULL, fileData->attrs);
+    //fprintf(stderr, "writer:%d:processed flush for reader:%d:reader_step:%d:writer_step:%d\n", fileData->rank, flushMsg->process_id, fileData->readerStep, fileData->writerStep);
 }
 
 void
 process_var_msg(FlexpathWriteFileData *fileData, Var_msg *varMsg)
 {
+    fp_verbose(fileData, "process Var msg for variable \"%s\"\n", varMsg->var_name);
     fileData->askedVars = add_var(fileData->askedVars, 
 				  strdup(varMsg->var_name), 
 				  NULL, 
 				  varMsg->process_id);
 }
 
+void
+drop_queued_data(FlexpathWriteFileData *fileData, int timestep)
+{
+    FlexpathQueueNode* node = threaded_dequeue(&fileData->dataQueue,
+					       &fileData->dataMutex,
+					       &fileData->dataCondition, 1);
+    FMfree_var_rec_elements(fileData->fm->ioFormat, node->data);
+
+    drop_evgroup_msg *dropMsg = malloc(sizeof(drop_evgroup_msg));
+    dropMsg->step = fileData->readerStep;
+    int wait = CMCondition_get(flexpathWriteData.cm, NULL);
+    dropMsg->condition = wait;
+    fp_verbose(fileData, "******* Triggering drop MSG\n");
+    EVsubmit_general(fileData->dropSource, dropMsg, drop_evgroup_msg_free, fileData->attrs);
+    //EVsubmit_general(fileData->dropSource, dropMsg, NULL, fileData->attrs);
+    // Will have to change when not using ctrl thread.
+    CMCondition_wait(flexpathWriteData.cm,  wait);
+    
+    fileData->readerStep++;
+}
+
 void
 process_open_msg(FlexpathWriteFileData *fileData, op_msg *open)
 {
+    fp_verbose(fileData, " Process Open msg, bridge %d, timestep %d\n", open->process_id, open->step);
     fileData->bridges[open->process_id].step = open->step;
     fileData->bridges[open->process_id].condition = open->condition;
     if (!fileData->bridges[open->process_id].created) {
@@ -1150,6 +1180,10 @@ process_open_msg(FlexpathWriteFileData *fileData, op_msg *open)
 			    fileData->bridges[open->process_id].myNum);
     }	
 	
+    if (open->step == fileData->readerStep + 1) {
+	drop_queued_data(fileData, fileData->readerStep);
+    }
+
     if (open->step == fileData->readerStep) {
 	pthread_mutex_lock(&fileData->openMutex);
 	fileData->openCount++;  
@@ -1169,37 +1203,59 @@ process_open_msg(FlexpathWriteFileData *fileData, op_msg *open)
 	log_error("Flexpath method control_thread: Received Past Step Open\n");
     } 
     else {
-	fp_write_log("STEP", "recieved op with future step\n");
+	fp_verbose(fileData, "received op with future step\n");
     }
 }
 
+void
+process_finalize_msg(FlexpathWriteFileData *fileData, op_msg *finalize)
+{
+    fp_verbose(fileData, " Process Finalize msg, bridge %d, timestep %d\n", finalize->process_id, finalize->step);
+	
+    FlexpathQueueNode* node = threaded_dequeue(&fileData->dataQueue,
+					       &fileData->dataMutex,
+					       &fileData->dataCondition, 1);
+    FMfree_var_rec_elements(fileData->fm->ioFormat, node->data);
+
+    drop_evgroup_msg *dropMsg = malloc(sizeof(drop_evgroup_msg));
+    dropMsg->step = fileData->readerStep;
+    int wait = CMCondition_get(flexpathWriteData.cm, NULL);
+    dropMsg->condition = wait;
+    fp_verbose(fileData, "******* Triggering drop MSG\n");
+    EVsubmit_general(fileData->dropSource, dropMsg, drop_evgroup_msg_free, fileData->attrs);
+    //EVsubmit_general(fileData->dropSource, dropMsg, NULL, fileData->attrs);
+    // Will have to change when not using ctrl thread.
+    CMCondition_wait(flexpathWriteData.cm,  wait);
+}
+
 void
 process_close_msg(FlexpathWriteFileData *fileData, op_msg *close)
 {
 
+    fp_verbose(fileData, " process close msg, bridge %d\n", close->process_id);
     pthread_mutex_lock(&fileData->openMutex);
     fileData->openCount--;
     fileData->bridges[close->process_id].opened=0;
     fileData->bridges[close->process_id].condition = close->condition;
     pthread_mutex_unlock(&fileData->openMutex);
 
-    if (fileData->openCount==0) {
-	FlexpathQueueNode* node = threaded_dequeue(&fileData->dataQueue, 
-						   &fileData->dataMutex, 
-						   &fileData->dataCondition, 1);
-	FMfree_var_rec_elements(fileData->fm->ioFormat, node->data);
+    /* if (fileData->openCount==0) { */
+    /* 	FlexpathQueueNode* node = threaded_dequeue(&fileData->dataQueue,  */
+    /* 						   &fileData->dataMutex,  */
+    /* 						   &fileData->dataCondition, 1); */
+    /* 	FMfree_var_rec_elements(fileData->fm->ioFormat, node->data); */
 
-	drop_evgroup_msg *dropMsg = malloc(sizeof(drop_evgroup_msg));
-	dropMsg->step = fileData->readerStep;
-	int wait = CMCondition_get(flexpathWriteData.cm, NULL);
-	dropMsg->condition = wait;
-	EVsubmit_general(fileData->dropSource, dropMsg, drop_evgroup_msg_free, fileData->attrs);
-	//EVsubmit_general(fileData->dropSource, dropMsg, NULL, fileData->attrs);
-	// Will have to change when not using ctrl thread.
-	CMCondition_wait(flexpathWriteData.cm,  wait); 		    
+    /* 	drop_evgroup_msg *dropMsg = malloc(sizeof(drop_evgroup_msg)); */
+    /* 	dropMsg->step = fileData->readerStep; */
+    /* 	int wait = CMCondition_get(flexpathWriteData.cm, NULL); */
+    /* 	dropMsg->condition = wait; */
+    /* 	EVsubmit_general(fileData->dropSource, dropMsg, drop_evgroup_msg_free, fileData->attrs); */
+    /* 	//EVsubmit_general(fileData->dropSource, dropMsg, NULL, fileData->attrs); */
+    /* 	// Will have to change when not using ctrl thread. */
+    /* 	CMCondition_wait(flexpathWriteData.cm,  wait); 		     */
 		     
-	fileData->readerStep++;
-    }
+    /* 	fileData->readerStep++; */
+    /* } */
 		
     op_msg *ack = malloc(sizeof(op_msg));
     ack->file_name = strdup(fileData->name);
@@ -1221,6 +1277,7 @@ var_handler(CManager cm, void *vevent, void *client_data, attr_list attrs)
 {
     FlexpathWriteFileData* fileData = (FlexpathWriteFileData*) client_data;
     Var_msg* msg = (Var_msg*) vevent;
+    fp_verbose(fileData, " var_msg received and queued\n");
     EVtake_event_buffer(cm, vevent);
     threaded_enqueue(&fileData->controlQueue, msg, VAR, 
 		     &fileData->controlMutex, &fileData->controlCondition, -1);
@@ -1233,16 +1290,20 @@ flush_handler(CManager cm, void* vevent, void* client_data, attr_list attrs)
     FlexpathWriteFileData* fileData = (FlexpathWriteFileData*) client_data;
     Flush_msg* msg = (Flush_msg*) vevent;
     int err = EVtake_event_buffer(cm, vevent);
+    fp_verbose(fileData, "flush_msg received and queued\n");
     threaded_enqueue(&fileData->controlQueue, msg, DATA_FLUSH, 
 		     &fileData->controlMutex, &fileData->controlCondition,
 		     -1);
+    //fprintf(stderr, "writer:%d:enqueued flush for reader:%d:reader_step:%d:writer_step:%d\n", fileData->rank, msg->process_id, fileData->readerStep, fileData->writerStep);
     return 0;
 }
 
 static int
 drop_evgroup_handler(CManager cm, void *vevent, void *client_data, attr_list attrs) {
+    FlexpathWriteFileData* fileData = (FlexpathWriteFileData*) client_data;
     drop_evgroup_msg *msg = vevent;
     // will have to change when not using control thread.
+    fp_verbose(fileData, "got drop evgroup message, signalling\n");
     CMCondition_signal(cm, msg->condition);    
     return 0;
 }
@@ -1253,12 +1314,19 @@ op_handler(CManager cm, void* vevent, void* client_data, attr_list attrs)
     FlexpathWriteFileData* fileData = (FlexpathWriteFileData*) client_data;
     op_msg* msg = (op_msg*) vevent;
     EVtake_event_buffer(cm, vevent);
-    if (msg->type == OPEN_MSG) {
+    fp_verbose(fileData, " op_msg received, message type %d\n", msg->type);
+    if(msg->type == OPEN_MSG) {
+	fp_verbose(fileData, " enqueueing open msg, bridge %d, step %d\n", msg->process_id, msg->step);
         threaded_enqueue(&fileData->controlQueue, msg, OPEN, 
 			 &fileData->controlMutex, &fileData->controlCondition, -1);
-    } else if (msg->type == CLOSE_MSG) {
+    } else if(msg->type == CLOSE_MSG) {
+	fp_verbose(fileData, " enqueueing close msg, bridge %d, step %d\n", msg->process_id, msg->step);
         threaded_enqueue(&fileData->controlQueue, msg, CLOSE, 
 			 &fileData->controlMutex, &fileData->controlCondition, -1);  			
+    } else if(msg->type == FINALIZE_MSG) {
+	fp_verbose(fileData, " enqueueing finalize msg, bridge %d\n", msg->process_id);
+        threaded_enqueue(&fileData->controlQueue, msg, FINALIZE, 
+			 &fileData->controlMutex, &fileData->controlCondition, -1);  			
     }
     return 0;
 }
@@ -1272,8 +1340,10 @@ control_thread(void *arg)
     FlexpathQueueNode *controlMsg;
     FlexpathQueueNode *dataNode;
     while (1) {
+//	fp_verbose(fileData, " Control thread waiting on msg\n");
 	if ((controlMsg = threaded_dequeue(&fileData->controlQueue, 
 	    &fileData->controlMutex, &fileData->controlCondition, 0))) {
+//	    fp_verbose(fileData, " Control thread got a msg\n");
 	    if (controlMsg->type==VAR) {
 		Var_msg *varMsg = (Var_msg*) controlMsg->data;
 		process_var_msg(fileData, varMsg);
@@ -1292,6 +1362,11 @@ control_thread(void *arg)
 		process_open_msg(fileData, open);                
 		EVreturn_event_buffer(flexpathWriteData.cm, open);
             }
+	    else if (controlMsg->type==FINALIZE) {
+                op_msg *open = (op_msg*) controlMsg->data;
+		process_finalize_msg(fileData, open);                
+		EVreturn_event_buffer(flexpathWriteData.cm, open);
+            }
 	    else if (controlMsg->type==CLOSE) {
                 op_msg* close = (op_msg*) controlMsg->data;
 		process_close_msg(fileData, close);
@@ -1332,6 +1407,30 @@ find_open_file(char* name)
 }
 
 
+void
+stone_close_handler(CManager cm, CMConnection conn, int closed_stone, void *client_data)
+{
+    FlexpathWriteFileData* file = flexpathWriteData.openFiles;
+    while (file) {
+	int i;
+	for (i=0; i < file->numBridges; i++) {
+	    if (file->bridges[i].myNum == closed_stone) {
+		int j;
+		file->bridges[i].opened = 0;
+		for (j=0; j< file->numBridges; j++) {
+		    if (file->bridges[j].opened == 1) {
+			/* if any bridge still open, simply return at this point, we're done */
+			return;
+		    }
+		}
+		/* no bridges in this file still open, drop all data */
+		drop_queued_data(file, -1);
+	    }
+	}
+        file = file->next;
+    }
+}
+
 // Initializes flexpath write local data structures
 extern void 
 adios_flexpath_init(const PairStruct *params, struct adios_method_struct *method) 
@@ -1346,9 +1445,15 @@ adios_flexpath_init(const PairStruct *params, struct adios_method_struct *method
     atom_t CM_TRANSPORT = attr_atom_from_string("CM_TRANSPORT");
     char * transport = getenv("CMTransport");
     if (transport == NULL) {
-	if (CMlisten(flexpathWriteData.cm) == 0) {
-	    fprintf(stderr, "error: unable to initialize connection manager.\n");
-	    exit(1);
+	int listened = 0;
+	while (listened == 0) {
+	    if (CMlisten(flexpathWriteData.cm) == 0) {
+		int rank;
+		MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+		fprintf(stderr, "error: writer %d:pid:%d unable to initialize connection manager. Trying again.\n", rank, (int)getpid());
+	    } else {
+		listened = 1;
+	    }
 	}
     } else {
 	attr_list listen_list = create_attr_list();
@@ -1365,6 +1470,8 @@ adios_flexpath_init(const PairStruct *params, struct adios_method_struct *method
     if (!forked) {
 	fprintf(stderr, "Wrtier error forking comm thread\n");
     }
+
+    EVregister_close_handler(flexpathWriteData.cm, stone_close_handler, &flexpathWriteData);
 }
 
 extern int 
@@ -1386,7 +1493,8 @@ adios_flexpath_open(struct adios_file_struct *fd,
     FlexpathWriteFileData *fileData = malloc(sizeof(FlexpathWriteFileData));
     mem_check(fileData, "fileData");
     memset(fileData, 0, sizeof(FlexpathWriteFileData));
-    
+    fp_verbose_init(fileData);
+
     fileData->maxQueueSize = 1;
     fileData->use_ctrl_thread = 1;
     if (method->parameters) {
@@ -1584,6 +1692,7 @@ adios_flexpath_write(
     FlexpathWriteFileData* fileData = find_open_file(method->group->name);
     FlexpathFMStructure* fm = fileData->fm;
 
+    fp_verbose(fileData, " adios_flexpath_write called\n");
     if (fm == NULL)
     {
 	log_error("adios_flexpath_write: something has gone wrong with format registration: %s\n", 
@@ -1667,6 +1776,7 @@ adios_flexpath_close(struct adios_file_struct *fd, struct adios_method_struct *m
     void *buffer = malloc(fileData->fm->size);    
     memcpy(buffer, fileData->fm->buffer, fileData->fm->size);
 
+    fp_verbose(fileData, " adios_flexpath_close called\n");
     threaded_enqueue(&fileData->dataQueue, buffer, 
 		     DATA_BUFFER,
 		     &fileData->dataMutex, 
@@ -1675,7 +1785,7 @@ adios_flexpath_close(struct adios_file_struct *fd, struct adios_method_struct *m
     
     // now gather offsets and send them via MPI to root
     struct adios_group_struct * g = fd->group;
-    struct adios_var_struct * list = g->vars;
+    struct adios_pg_struct * pg = fd->pgs_written;
     evgroup *gp = malloc(sizeof(evgroup));    
     gp->group_name = strdup(method->group->name);
     gp->process_id = fileData->rank;
@@ -1695,56 +1805,61 @@ adios_flexpath_close(struct adios_file_struct *fd, struct adios_method_struct *m
 	int myrank = fileData->rank;
 	int commsize = fileData->size;
 
-	while (list) {
-	    char *fullname = resolve_path_name(list->path, list->name);
-	    //int num_local_offsets = 0;
-	    uint64_t *local_offsets = NULL;
-	    uint64_t *local_dimensions = NULL;
-	    uint64_t *global_dimensions = NULL; // same at each rank.
-	    int num_local_offsets = get_var_offsets(list, g, 
-						    &local_offsets, 
-						    &local_dimensions, 
-						    &global_dimensions);
-	    // flip for fortran here.
-            if (fileData->host_language == FP_FORTRAN_MODE) {
-                reverse_dims(local_offsets, num_local_offsets);
-                reverse_dims(local_dimensions, num_local_offsets);
-                reverse_dims(global_dimensions, num_local_offsets);
+        while (pg) {
+            struct adios_var_struct * list = pg->vars_written;
+            while (list) {
+                char *fullname = resolve_path_name(list->path, list->name);
+                //int num_local_offsets = 0;
+                uint64_t *local_offsets = NULL;
+                uint64_t *local_dimensions = NULL;
+                uint64_t *global_dimensions = NULL; // same at each rank.
+                int num_local_offsets = get_var_offsets(list, g, 
+                        &local_offsets, 
+                        &local_dimensions, 
+                        &global_dimensions);
+                // flip for fortran here.
+                if (fileData->host_language == FP_FORTRAN_MODE) {
+                    reverse_dims(local_offsets, num_local_offsets);
+                    reverse_dims(local_dimensions, num_local_offsets);
+                    reverse_dims(global_dimensions, num_local_offsets);
+                }
+
+                if (num_local_offsets > 0) {
+                    uint64_t *all_offsets = NULL;
+                    uint64_t *all_local_dims = NULL;
+
+                    int buf_size = num_local_offsets * commsize * sizeof(uint64_t);		    
+                    all_offsets = malloc(buf_size);		
+                    all_local_dims = malloc(buf_size);
+
+                    int arr_size = num_local_offsets * sizeof(uint64_t);
+                    MPI_Allgather(local_offsets, arr_size, MPI_BYTE, 
+                            all_offsets, arr_size, MPI_BYTE,
+                            fileData->mpiComm);
+
+                    MPI_Allgather(local_dimensions, arr_size, MPI_BYTE, 
+                            all_local_dims, arr_size, MPI_BYTE,
+                            fileData->mpiComm);
+
+                    num_gbl_vars++;
+                    offset_struct *ostruct = malloc(sizeof(offset_struct));
+                    ostruct->offsets_per_rank = num_local_offsets;
+                    ostruct->total_offsets = num_local_offsets * commsize;
+                    ostruct->local_offsets = all_offsets;
+                    ostruct->local_dimensions = all_local_dims;
+                    ostruct->global_dimensions = global_dimensions;
+
+                    gbl_vars = realloc(gbl_vars, sizeof(global_var) * num_gbl_vars);
+                    gbl_vars[num_gbl_vars - 1].name = fullname;
+                    gbl_vars[num_gbl_vars - 1].noffset_structs = 1;
+                    gbl_vars[num_gbl_vars - 1].offsets = ostruct;
+
+                }
+                list=list->next;
             }
 
-	    if (num_local_offsets > 0) {
-		uint64_t *all_offsets = NULL;
-		uint64_t *all_local_dims = NULL;
-		
-		int buf_size = num_local_offsets * commsize * sizeof(uint64_t);		    
-		all_offsets = malloc(buf_size);		
-		all_local_dims = malloc(buf_size);
-
-		int arr_size = num_local_offsets * sizeof(uint64_t);
-		MPI_Allgather(local_offsets, arr_size, MPI_BYTE, 
-			      all_offsets, arr_size, MPI_BYTE,
-			      fileData->mpiComm);
-
-		MPI_Allgather(local_dimensions, arr_size, MPI_BYTE, 
-			      all_local_dims, arr_size, MPI_BYTE,
-			      fileData->mpiComm);
-		
-		num_gbl_vars++;
-		offset_struct *ostruct = malloc(sizeof(offset_struct));
-		ostruct->offsets_per_rank = num_local_offsets;
-		ostruct->total_offsets = num_local_offsets * commsize;
-		ostruct->local_offsets = all_offsets;
-		ostruct->local_dimensions = all_local_dims;
-		ostruct->global_dimensions = global_dimensions;
-
-		gbl_vars = realloc(gbl_vars, sizeof(global_var) * num_gbl_vars);
-		gbl_vars[num_gbl_vars - 1].name = fullname;
-		gbl_vars[num_gbl_vars - 1].noffset_structs = 1;
-		gbl_vars[num_gbl_vars - 1].offsets = ostruct;
-
-	    }
-	    list=list->next;
-	}
+            pg = pg->next;
+        }
 
 	gp->num_vars = num_gbl_vars;
 	gp->step = fileData->writerStep;
@@ -1769,7 +1884,8 @@ extern void
 adios_flexpath_finalize(int mype, struct adios_method_struct *method) 
 {
     FlexpathWriteFileData* fileData = flexpathWriteData.openFiles;
-    while (fileData) {
+    fp_verbose(fileData, "adios_flexpath_finalize called\n");
+    while(fileData) {
 
 	update_step_msg *stepmsg = malloc(sizeof(update_step_msg));
 	stepmsg->finalized = 1;
@@ -1779,6 +1895,7 @@ adios_flexpath_finalize(int mype, struct adios_method_struct *method)
 
         pthread_mutex_lock(&fileData->dataMutex);
         while (fileData->dataQueue != NULL) {
+	    fp_verbose(fileData, " Wait in flexpath finalize\n");
 	    pthread_cond_wait(&fileData->dataCondition, &fileData->dataMutex);
 	}
 	pthread_mutex_unlock(&fileData->dataMutex);
@@ -1789,10 +1906,17 @@ adios_flexpath_finalize(int mype, struct adios_method_struct *method)
 }
 
 // provides unknown functionality
-extern enum ADIOS_FLAG 
+extern enum BUFFERING_STRATEGY 
 adios_flexpath_should_buffer (struct adios_file_struct * fd,struct adios_method_struct * method) 
 {
-    return adios_flag_no;
+    return no_buffering;  
+}
+
+extern void 
+adios_flexpath_buffer_overflow (struct adios_file_struct * fd, 
+                                struct adios_method_struct * method)
+{
+    // this call never happens without shared buffering
 }
 
 // provides unknown functionality
@@ -1839,7 +1963,7 @@ adios_flexpath_get_write_buffer(struct adios_file_struct *fd,
         *buffer = malloc (*size);
         if (!*buffer) {        
             adios_method_buffer_free (mem_allowed);
-            log_error ("ERROR: Out of memory allocating %llu bytes for %s in %s:%s()\n"
+            log_error ("ERROR: Out of memory allocating %" PRIu64 " bytes for %s in %s:%s()\n"
                     ,*size, v->name, __FILE__, __func__
                     );
             v->got_buffer = adios_flag_no;
@@ -1858,7 +1982,7 @@ adios_flexpath_get_write_buffer(struct adios_file_struct *fd,
     }
     else {    
         adios_method_buffer_free (mem_allowed);
-        log_error ("OVERFLOW: Cannot allocate requested buffer of %llu "
+        log_error ("OVERFLOW: Cannot allocate requested buffer of %" PRIu64 
                          "bytes for %s in %s:%s()\n"
                 ,*size
                 ,v->name
