@@ -119,13 +119,13 @@ int main (int argc, char ** argv)
 
     alloc_vars();
     adios_init_noxml (comm);
-    adios_allocate_buffer (ADIOS_BUFFER_ALLOC_NOW, 4400);
+    adios_set_max_buffer_size (4400);
     err = adios_read_init_method(ADIOS_READ_METHOD_BP, comm, "verbose=2");
     if (err) {
         printE ("%s\n", adios_errmsg());
     }
 
-    adios_declare_group (&m_adios_group, "bigdata", "iter", adios_flag_yes);
+    adios_declare_group (&m_adios_group, "bigdata", "iter", adios_stat_default);
     adios_select_method (m_adios_group, "MPI", "", "");
 
 
