@@ -171,7 +171,7 @@ int main (int argc, char ** argv)
     alloc_vars();
     if (do_write) {
         adios_init_noxml (comm);
-        adios_allocate_buffer (ADIOS_BUFFER_ALLOC_NOW, 10);
+        adios_set_max_buffer_size (10);
     }
     if (do_read) {
         err = adios_read_init_method(read_method, comm, "verbose=2");
@@ -182,7 +182,7 @@ int main (int argc, char ** argv)
 
     
     if (do_write) {
-        adios_declare_group (&m_adios_group, "selections", "iter", adios_flag_yes);
+        adios_declare_group (&m_adios_group, "selections", "iter", adios_stat_default);
         adios_select_method (m_adios_group, write_method, "verbose=2", "");
 
         define_vars();
