@@ -38,7 +38,7 @@ UTILS_DIR="$TRUNKDIR/utils"
 DATASET_BUILDER_EXE_BASENAME="build_standard_dataset"
 QUERY_SEQSCAN_EXE_BASENAME="compute_expected_query_results"
 QUERY_EXE_BASENAME="adios_query_test"
-LIST_METHODS_EXE_BASENAME="list_methods"
+LIST_METHODS_EXE_BASENAME="adios_list_methods"
 
 DATASET_BUILDER_EXE_PATH="$TEST_PROGRAMS_DIR/$DATASET_BUILDER_EXE_BASENAME"
 QUERY_SEQSCAN_EXE_PATH="$QUERY_COMMON_DIR/$QUERY_SEQSCAN_EXE_BASENAME"
@@ -86,18 +86,9 @@ done
 ALL_QUERY_ENGINES=$( \
   $MPIRUN_SERIAL $LIST_METHODS_EXE_PATH |
   awk '
-    /^Available/{
-      transforms = ($2 == "query")
-    }
-    {
-      if (transforms) {
-        if (skippedheader) {
-          gsub("ADIOS_QUERY_METHOD_","",$1)
-          print $1
-        } else {
-          skippedheader = 1
-        }
-      }
+    /ADIOS_QUERY_METHOD/{
+      gsub("ADIOS_QUERY_METHOD_","",$1)
+      print $1
     }
   ' |
   tr "A-Z\n" 'a-z '
@@ -121,6 +112,7 @@ fi
 # NO FURTHER CONFIGURATION VARIABLES BELOW THIS POINT
 #
 
+N_INVALID_RESULTS=0
 function init_work_directory() {
   echo "STEP 1: INITIALIZING THE TEST WORKING DIRECTORY..."
   
@@ -180,6 +172,15 @@ function build_indexed_datasets_fastbit() {
   set +o xtrace
 }
 
+function build_indexed_datasets_minmax() {
+  local DSID="$1"
+  local DSOUTPUT="$2"
+  [[ $# -eq 2 ]] || die "ERROR: Internal testing error, invalid parameters to build_indexed_datasets_minmax: $@"
+  
+  invoke_dataset_builder "$DSID" "$DSOUTPUT" "none"
+  echo "Minmax method uses data as is, no index file is built"
+}
+
 function build_datasets() {
   echo "STEP 2: INDEXING ALL TEST DATASETS USING ALL ENABLED INDEXING METHODS"
   echo "(ALSO PRODUCING A NON-INDEXED VERSION OF EACH DATASET FOR REFERENCE)"
@@ -237,9 +238,11 @@ function query_datasets() {
           echo "====== COMPUTING EXPECTED OUTPUT OF QUERY $QUERY_NAME ON DATASET $DSID IN $FILEMODE MODE ======"
           echo
           set -o xtrace
-          $MPIRUN_SERIAL "$QUERY_SEQSCAN_EXE_LOCAL" "$NOINDEX_DS" "$QUERY_XML_LOCAL" "$FILEMODE" > "$EXPECTED_POINTS_FILE" ||
+          $MPIRUN_SERIAL "$QUERY_SEQSCAN_EXE_LOCAL" "$NOINDEX_DS" "$QUERY_XML_LOCAL" "$FILEMODE" > "$EXPECTED_POINTS_FILE".raw ||
             die "ERROR: $QUERY_SEQSCAN_EXE_LOCAL failed with exit code $?"
           set +o xtrace
+          # filter out job manager's output line at the end of stdout
+          grep -v "^Application" "$EXPECTED_POINTS_FILE".raw > "$EXPECTED_POINTS_FILE"
         
           # NOTE: the sequential scan program produces a point list that is guaranteed to be sorted in C array order, so no need to sort it here
         done
@@ -251,6 +254,10 @@ function query_datasets() {
 
         # Run the query for each query engine implementation and compare to the expected results
         for QUERY_ENGINE in $ALL_QUERY_ENGINES; do
+          case $QUERY_ENGINE in *minmax*) 
+              echo --- Skip evaluating with MINMAX method ---; 
+              continue 
+          esac
           local QE_WORKDIR="./$QUERY_ENGINE"
 
           for INDEXED_DS in "$QE_WORKDIR/$DSID.$QUERY_ENGINE".*bp; do
@@ -265,9 +272,12 @@ function query_datasets() {
               echo "====== RUNNING QUERY $QUERY_NAME USING QUERY ENGINE $QUERY_ENGINE ON DATASET $DSID IN $FILEMODE MODE ======"
               echo
               set -o xtrace
-              $MPIRUN_SERIAL "$QUERY_EXE_LOCAL" "$INDEXED_DS" "$QUERY_XML_LOCAL" "$QUERY_ENGINE" "$FILEMODE" > "$OUTPUT_POINTS_FILE" ||
+              $MPIRUN_SERIAL "$QUERY_EXE_LOCAL" "$INDEXED_DS" "$QUERY_XML_LOCAL" "$QUERY_ENGINE" "$FILEMODE" > "$OUTPUT_POINTS_FILE".raw ||
                 die "ERROR: $QUERY_EXE_LOCAL failed with exit code $?"
               set +o xtrace
+              # filter out job manager's output line at the end of stdout
+              grep -v "^Application" "$OUTPUT_POINTS_FILE".raw > "$OUTPUT_POINTS_FILE"
+
 
               # Sort the output points in C array order, since the query engine makes no guarantee as to the ordering of the results
               # Sort file in place (-o FILE) with numerical sort order (-n) on each of the first 9 fields (-k1,1 ...)
@@ -280,7 +290,7 @@ function query_datasets() {
                 echo "ERROR: ADIOS Query does not return the expected points matching query $QUERY_NAME on dataset $DSID in $FILEMODE mode using query engine $QUERY_ENGINE"
                 echo "Compare \"$EXPECTED_POINTS_FILE\" (expected points) vs. \"$OUTPUT_POINTS_FILE\" (returned points) in \"$PWD\""
                 echo "The BP file queried is \"$INDEXED_DS\" and the query is specified by \"$QUERY_XML_LOCAL\""
-                exit 1  
+                let "N_INVALID_RESULTS=N_INVALID_RESULTS+1"  
               fi
             done
           done
@@ -290,8 +300,10 @@ function query_datasets() {
 }
 
 # FINALLY, CALL THE FUNCTIONS IN SEQUENCE
-echo "NOTE: Testing query methods: $ALL_QUERY_METHODS"
-echo "NOTE: Testing on datasets: $ALL_DATASETS"
+echo "NOTE: Testing query methods: $ALL_QUERY_ENGINES"
+echo "NOTE: Testing on datasets: $ALL_DATASET_IDS"
 init_work_directory
 build_datasets
 query_datasets
+echo "NOTE: Number of tests that ran but produced a result different from the expected : $N_INVALID_RESULTS"
+exit $N_INVALID_RESULTS
