@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <stdint.h>
 #include <unistd.h>
+#include <inttypes.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -164,7 +165,7 @@ void display_help() {
             "  --help      | -h           Print this help.\n"
             "  --verbose   | -v           Print log about what this program is doing.\n"
             "                               Use multiple -v to increase logging level.\n"
-            "Typical use: bpls -lavr <file>\n"
+            "Typical use: bpls -lav <file>\n"
             );
 }
 
@@ -452,7 +453,7 @@ void print_file_size(uint64_t size)
     }
     if (r > 511)
         s++;
-    printf ("  file size:     %lld %s\n", s, sm[idx]); 
+    printf ("  file size:     %" PRId64 " %s\n", s, sm[idx]);
 }
 
 
@@ -528,6 +529,7 @@ int doList_group (ADIOS_FILE *fp)
             if (retval) {
                 fprintf(stderr, "Error: %s\n", adios_errmsg());
             }
+            free(value);
         }
         len = strlen(adios_type_to_string(vartype));
         if (len > maxtypelen) maxtypelen = len;
@@ -586,9 +588,9 @@ int doList_group (ADIOS_FILE *fp)
                 if (timed) 
                     fprintf(outf, "%d*", vi->nsteps);
                 if (vi->ndim > 0) {
-                    fprintf(outf,"{%lld", vi->dims[0]);
+                    fprintf(outf,"{%" PRId64, vi->dims[0]);
                     for (j=1; j < vi->ndim; j++) {
-                        fprintf(outf,", %lld", vi->dims[j]);
+                        fprintf(outf,", %" PRId64, vi->dims[j]);
                     }
                     fprintf(outf,"}");
                 } else {
@@ -742,21 +744,33 @@ int doList_group (ADIOS_FILE *fp)
     }
     free(names);
     free(isVar);
+    free(vis);
     return 0;
 }                
 
 #define PRINT_ARRAY(str, ndim, dims, loopvar, format) \
     fprintf(outf,"%s",str); \
     if (ndim > 0) { \
-        fprintf(outf,"{"#format, dims[0]); \
+        fprintf(outf,"{%"#format, dims[0]); \
         for (loopvar=1; loopvar < ndim; loopvar++) { \
-            fprintf(outf,", "#format, dims[loopvar]); \
+            fprintf(outf,", %"#format, dims[loopvar]); \
         } \
         fprintf(outf,"}\n"); \
     } else { \
         fprintf(outf,"empty\n"); \
     }
 
+#define PRINT_ARRAY64(str, ndim, dims, loopvar) \
+    fprintf(outf,"%s",str); \
+    if (ndim > 0) { \
+        fprintf(outf,"{%" PRId64, dims[0]); \
+        for (loopvar=1; loopvar < ndim; loopvar++) { \
+            fprintf(outf,", %" PRId64, dims[loopvar]); \
+        } \
+        fprintf(outf,"}\n"); \
+    } else { \
+        fprintf(outf,"empty\n"); \
+    }
 
 void printMeshes (ADIOS_FILE  *fp)
 {
@@ -784,39 +798,40 @@ void printMeshes (ADIOS_FILE  *fp)
                adios_complete_meshinfo (fp, meshfp, mi); 
             }
             fprintf(outf, "    type:         ");
+
             switch (mi->type) {
                 case ADIOS_MESH_UNIFORM:
                     fprintf(outf, "uniform\n");
-                    PRINT_ARRAY("    dimensions:   ", 
+                    PRINT_ARRAY64("    dimensions:   ",
                                  mi->uniform->num_dimensions, 
                                  mi->uniform->dimensions, 
-                                 j, %lld)
+                                 j)
                     if (mi->uniform->origins) {
                         PRINT_ARRAY("    origins:      ", 
                                      mi->uniform->num_dimensions, 
                                      mi->uniform->origins, 
-                                     j, %g) 
+                                     j, g)
                     }
                     if (mi->uniform->spacings) {
                         PRINT_ARRAY ("    spacings:     ", 
                                      mi->uniform->num_dimensions, 
                                      mi->uniform->spacings,
-                                     j, %g)
+                                     j, g)
                     }
                     if (mi->uniform->maximums) {
                         PRINT_ARRAY ("    maximums:     ", 
                                      mi->uniform->num_dimensions, 
                                      mi->uniform->maximums,
-                                     j, %g)
+                                     j, g)
                     }
                     break;
 
                 case ADIOS_MESH_RECTILINEAR:
                     fprintf(outf, "rectilinear\n");
-                    PRINT_ARRAY("    dimensions:   ", 
+                    PRINT_ARRAY64("    dimensions:   ",
                                 mi->rectilinear->num_dimensions, 
                                 mi->rectilinear->dimensions, 
-                                j, %lld)
+                                j)
                     if (mi->rectilinear->use_single_var) {
                         fprintf(outf, "    coordinates:  single-var: \"%s\"\n", 
                                 mi->rectilinear->coordinates[0]);
@@ -832,10 +847,10 @@ void printMeshes (ADIOS_FILE  *fp)
 
                 case ADIOS_MESH_STRUCTURED:
                     fprintf(outf, "structured\n");
-                    PRINT_ARRAY("    dimensions:   ", 
+                    PRINT_ARRAY64("    dimensions:   ",
                                 mi->structured->num_dimensions, 
                                 mi->structured->dimensions, 
-                                j, %lld);
+                                j);
                     if (mi->structured->use_single_var) {
                         fprintf(outf, "    points:       single-var: \"%s\"\n", 
                                 mi->structured->points[0]);
@@ -853,7 +868,7 @@ void printMeshes (ADIOS_FILE  *fp)
                 case ADIOS_MESH_UNSTRUCTURED:
                     fprintf(outf, "unstructured\n");
                     if (mi->unstructured->nvar_points <= 1) {
-                        fprintf(outf, "    npoints:      %lld\n", mi->unstructured->npoints);
+                        fprintf(outf, "    npoints:      %" PRId64 "\n", mi->unstructured->npoints);
                         fprintf(outf, "    points:       single-var: \"%s\"\n", 
                                 mi->unstructured->points[0]);
                     } else {
@@ -868,7 +883,7 @@ void printMeshes (ADIOS_FILE  *fp)
                     for (i=0; i < mi->unstructured->ncsets; i++) { 
                         fprintf(outf, "    cell set %d:\n", i);
                         fprintf(outf, "      cell type:  %d\n", mi->unstructured->ctypes[i]);
-                        fprintf(outf, "      ncells:     %llu\n", mi->unstructured->ccounts[i]);
+                        fprintf(outf, "      ncells:     %" PRIu64 "\n", mi->unstructured->ccounts[i]);
                         fprintf(outf, "      cells var:  \"%s\"\n", mi->unstructured->cdata[i]);
                     }
                     fprintf(outf, "    nspaces:      %d\n", mi->unstructured->nspaces);
@@ -1148,7 +1163,7 @@ int getTypeInfo( enum ADIOS_DATATYPES adiosvartype, int* elemsize)
 }
 
 #define PRINT_DIMS64(str, v, n, loopvar) printf("%s = { ", str); \
-    for (loopvar=0; loopvar<n;loopvar++) printf("%lld ", v[loopvar]);    \
+    for (loopvar=0; loopvar<n;loopvar++) printf("%" PRId64 " ", v[loopvar]);    \
 printf("}")
 /** Read data of a variable and print 
  * Return: 0: ok, != 0 on error
@@ -1170,7 +1185,6 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
     int  readn[MAX_DIMS];   // how big chunk to read in in each dimension?
     int  status;            
     bool incdim;            // used in incremental reading in
-    ADIOS_SELECTION * sel;  // boundnig box to read
     int ndigits_dims[32];        // # of digits (to print) of each dimension 
 
     if (getTypeInfo(vi->type, &elemsize)) {
@@ -1196,13 +1210,13 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
             ct = icount[0];
 
         if (verbose>2) 
-            printf("    j=0, st=%llu ct=%llu\n", st, ct);
+            printf("    j=0, st=%" PRIu64 " ct=%" PRIu64 "\n", st, ct);
 
         start_t[0] = st;
         count_t[0] = ct;
         nelems *= ct;
         if (verbose>1) 
-            printf("    s[0]=%llu, c[0]=%llu, n=%llu\n", 
+            printf("    s[0]=%" PRIu64 ", c[0]=%" PRIu64 ", n=%" PRIu64 "\n",
                     start_t[0], count_t[0], nelems);
         
         tidx = 1;
@@ -1220,18 +1234,18 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
             ct = icount[j+tidx];
 
         if (verbose>2) 
-            printf("    j=%d, st=%llu ct=%llu\n", j+tidx, st, ct);
+            printf("    j=%d, st=%" PRIu64 " ct=%" PRIu64 "\n", j+tidx, st, ct);
 
         start_t[j+tidx] = st;
         count_t[j+tidx] = ct;
         nelems *= ct;
         if (verbose>1) 
-            printf("    s[%d]=%llu, c[%d]=%llu, n=%llu\n", 
+            printf("    s[%d]=%" PRIu64 ", c[%d]=%" PRIu64 ", n=%" PRIu64 "\n",
                     j+tidx, start_t[j+tidx], j+tidx, count_t[j+tidx], nelems);
     }
 
     if (verbose>1) {
-        printf(" total size of data to read = %llu\n", nelems*elemsize);
+        printf(" total size of data to read = %" PRIu64 "\n", nelems*elemsize);
     }
 
     print_slice_info(vi->ndim, vi->dims, timed, vi->nsteps, start_t, count_t);
@@ -1271,7 +1285,7 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
         sum = sum * (uint64_t) count_t[i];
         actualreadn = actualreadn * readn[i];
     }
-    if (verbose>1) printf("    read %d elements at once, %lld in total (nelems=%lld)\n", actualreadn, sum, nelems);
+    if (verbose>1) printf("    read %d elements at once, %" PRId64 " in total (nelems=%" PRId64 ")\n", actualreadn, sum, nelems);
 
 
     // init s and c
@@ -1300,7 +1314,7 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
         }
 
         // read a slice finally
-        sel = adios_selection_boundingbox (vi->ndim, s+tidx, c+tidx);
+        ADIOS_SELECTION *sel = adios_selection_boundingbox (vi->ndim, s+tidx, c+tidx);
         if (timed) {
             status = adios_schedule_read_byid (fp, sel, vi->varid, s[0], c[0], data); 
         } else {
@@ -1314,13 +1328,14 @@ int readVar(ADIOS_FILE *fp, ADIOS_VARINFO *vi, const char * name, bool timed)
         }
 
         status = adios_perform_reads (fp, 1); // blocking read performed here
+        adios_selection_delete (sel);
         if (status < 0) {
             fprintf(stderr, "Error when reading variable %s. errno=%d : %s \n", name, adios_errno, adios_errmsg());
             free(data);
             return 11;
         }
 
-        //if (verbose>2) printf("  read %lld bytes\n", bytes_read);
+        //if (verbose>2) printf("  read %" PRId64 " bytes\n", bytes_read);
 
         // print slice
         print_dataset(data, vi->type, s, c, tdims, ndigits_dims); 
@@ -1464,9 +1479,9 @@ void print_slice_info(int ndim, uint64_t *dims, int timed, int nsteps, uint64_t
             isaslice = true;
     }
     if (isaslice) {
-        fprintf(outf,"%c   slice (%lld:%lld", commentchar, s[0], s[0]+c[0]-1);
+        fprintf(outf,"%c   slice (%" PRId64 ":%" PRId64, commentchar, s[0], s[0]+c[0]-1);
         for (i=1; i<tdim; i++) {
-            fprintf(outf,", %lld:%lld", s[i], s[i]+c[i]-1);
+            fprintf(outf,", %" PRId64 ":%" PRId64, s[i], s[i]+c[i]-1);
         }
         fprintf(outf,")\n");
     }
@@ -1731,9 +1746,9 @@ int print_dataset(void *data, enum ADIOS_DATATYPES adiosvartype,
         idxstr[0] = '\0'; // empty idx string
         if (nextcol == 0) {
             if (!noindex && tdims > 0) {
-                sprintf(idxstr,"    (%*lld",ndigits[0], ids[0]);
+                sprintf(idxstr,"    (%*" PRId64,ndigits[0], ids[0]);
                 for (i=1; i<tdims; i++) {
-                    sprintf(buf,",%*lld",ndigits[i],ids[i]);
+                    sprintf(buf,",%*" PRId64,ndigits[i],ids[i]);
                     strcat(idxstr, buf);
                 }
                 strcat(idxstr,")    ");
@@ -1835,7 +1850,7 @@ void print_decomp(ADIOS_VARINFO *vi)
                 }
                 for (k=0; k < vi->ndim; k++) {
                     if (vi->blockinfo[blockid].count[k]) {
-                    fprintf(outf, "%*lld:%*lld", 
+                    fprintf(outf, "%*" PRId64 ":%*" PRId64,
                             ndigits_dims[k],
                             vi->blockinfo[blockid].start[k],
                             ndigits_dims[k],
